@@ -4,14 +4,14 @@ use warnings;
 package UR::DataSource::RDBMS::BitmapIndex;
 
 use UR;
-our $VERSION = "0.43"; # UR $VERSION;
+our $VERSION = "0.44"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::BitmapIndex',
     is => ['UR::DataSource::RDBMS::Entity'],
     dsmap => 'dd_bitmap_index',
     er_role => '',
-    id_properties => [qw/data_source owner table_name bitmap_index_name/],
+    id_properties => [qw/data_source table_name bitmap_index_name/],
     properties => [
         bitmap_index_name                => { type => 'varchar', len => undef, sql => 'bitmap_index_name' },
         data_source                      => { type => 'varchar', len => undef, sql => 'data_source' },
