@@ -4,14 +4,14 @@ use warnings;
 package UR::DataSource::RDBMS::FkConstraintColumn;
 
 use UR;
-our $VERSION = "0.43"; # UR $VERSION;
+our $VERSION = "0.44"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::FkConstraintColumn',
     is => ['UR::DataSource::RDBMS::Entity'],
     dsmap => 'dd_fk_constraint_column',
     er_role => 'bridge',
-    id_properties => [qw/data_source owner table_name fk_constraint_name column_name/],
+    id_properties => [qw/data_source table_name fk_constraint_name column_name/],
     properties => [
         column_name                      => { type => 'varchar', len => undef, sql => 'column_name' },
         data_source                      => { type => 'varchar', len => undef, sql => 'data_source' },
@@ -19,7 +19,6 @@ UR::Object::Type->define(
         namespace                        => { calculate_from => [ 'data_source'],
                                               calculate => q( (split(/::/,$data_source))[0] ) },
         fk_constraint_name               => { type => 'varchar', len => undef, sql => 'fk_constraint_name' },
-        owner                            => { type => 'varchar', len => undef, is_optional => 1, sql => 'owner' },
         table_name                       => { type => 'varchar', len => undef, sql => 'table_name' },
         r_column_name                    => { type => 'varchar', len => undef, sql => 'r_column_name' },
         r_table_name                     => { type => 'varchar', len => undef, sql => 'r_table_name' },
