@@ -4,7 +4,7 @@ package UR::Namespace::Command::Update::ClassesFromDb;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.41"; # UR $VERSION;
+our $VERSION = "0.44"; # UR $VERSION;
 use Text::Diff;
 
 UR::Object::Type->define(
@@ -104,7 +104,8 @@ sub execute {
         if ($specified_class_name_arrayref) {
             $ds_table_list = [
                 map { [$_->data_source, $_->table_name] }
-                map { $_->__meta__ } 
+                grep { $_->data_source }
+                map { $_->__meta__ }
                 @$specified_class_name_arrayref
             ];        
         }
@@ -432,9 +433,9 @@ sub _update_database_metadata_objects_for_schema_changes {
     my @current_table_names = $data_source->_get_table_names_from_data_dictionary();
     my %current_table_names = map { s/"|'//g; $_ => $_ } @current_table_names;
 
-    my %all_table_names = (%current_table_names, %previous_table_names);
-
-    if($table_name) { %all_table_names = ($table_name => 1) }
+    my %all_table_names = $table_name
+                            ? ( $table_name => 1 )
+                            : ( %current_table_names, %previous_table_names);
 
     my $new_object_revision = $UR::Context::current->now();
 
@@ -449,8 +450,6 @@ sub _update_database_metadata_objects_for_schema_changes {
         my $last_recorded_ddl_time;
         my $last_object_revision;
 
-        # UR always keeps table names stored in upper-case.  Some databases (mysql)
-        # are case sensitive when querying the data dictionary
         my $db_table_name = $current_table_names{$table_name};
 
         eval {
@@ -584,50 +583,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
     my $namespace = $self->namespace_name;
 
-=cut
-
-    $self->status_message("Using filesystem classes for namespace \"$namespace\" (this may be slow)");
-    my @material_classes = $namespace->get_material_classes;
-
-
-    $self->status_message("Verifying class/table relationships...");
-    my %table_ids_used;
-    for my $class (sort { $a->class_name cmp $b->class_name } @material_classes) {
-        my $table_name  = $class->table_name;
-        next unless $table_name;
-
-        my $class_name  = $class->class_name;
-
-        if (my $prev_class_name = $table_ids_used{$table_name}) {
-            $self->error_message(
-                sprintf(
-                    "C %-40s uses table %-32s, but so does %-40s" . "\n",
-                    $class_name, $table_name, $prev_class_name
-                )
-            );
-            return;
-        }
-
-        my $data_source = $class->data_source;
-
-        my $table = UR::DataSource::RDBMS::Table->get(data_source => $data_source, table_name => $table_name)
-                    ||
-                    UR::DataSource::RDBMS::Table::Ghost->get(data_source => $data_source, table_name => $table_name);
-
-        unless ($table) {
-            $self->error_message(
-                sprintf(
-                    "C %-32s %-32s is referenced by class %-40s but cannot be found!?" . "\n",
-                    $data_source, $table_name, $class_name
-                )
-            );
-            return;
-        }
-        $table_ids_used{$table_name} = $class;
-    }
-
-=cut
-
     $self->status_message("Updating classes...");
 
     my %dd_changes_by_class = (
@@ -668,17 +623,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         }
 
         my $table = $fk->get_table;
-        # FIXME should this use $data_source->get_class_meta_for_table($table) instead?
-        my $class = 
-            UR::Object::Type->get(
-                data_source_id => $table->data_source,
-                table_name     => $table->table_name,
-            )
-            ||
-            UR::Object::Type::Ghost->get(
-                data_source_id => $table->data_source,
-                table_name     => $table->table_name,
-            );
+        my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
+                                                          table_name  => $table->table_name);
 
         unless ($class) {
             ##$DB::single = 1;
@@ -714,11 +660,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         }
         my $column_name = $column->column_name;
 
-        # FIXME should this use $data_source->get_class_meta_for_table($table) instead?
-        my $class = UR::Object::Type->get(
-            data_source_id => $table->data_source,
-            table_name     => $table->table_name,
-        );
+        my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
+                                                          table_name  => $table->table_name);
         unless ($class) {
             $self->status_message(sprintf("~ No class found for deleted column %-32s %-32s\n", $table->table_name, $column_name));
             next;
@@ -853,6 +796,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         else {
             # create
             my $data_source = $table->data_source;
+            my $data_source_id = (ref $data_source ? $data_source->id : $data_source);
             my $class_name = $data_source->resolve_class_name_for_table_name($table_name,$table->table_type);
             unless ($class_name) {
                 Carp::confess(
@@ -865,12 +809,13 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             # new one actually has a table, then this is just another schema change and
             # not an error.  Set the table_name attribute and go on...
             my $class = UR::Object::Type->get(class_name => $class_name);
-            my $prev_table_name = $class->table_name if ($class);
+            my $prev_table_name = ($class ? $class->table_name : undef);
+            my $prev_data_source_id = ($class ? $class->data_source_id : undef);
             if ($class && $prev_table_name) {
 
                 Carp::confess(
-                    "Class $class_name already exists for table '$prev_table_name'."
-                    . "  Cannot generate class for $table_name."
+                    "Class $class_name already exists for table '$prev_table_name' in $prev_data_source_id."
+                    . "  Cannot generate class for $table_name in $data_source_id."
                 );
             }
 
@@ -904,15 +849,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                     );
                 }
             }
-
-            #unless ($class->class_name->isa('UR::Entity')) {
-            #    my $inheritance = UR::Object::Inheritance->create(
-            #        class_name => $class->class_name,
-            #        parent_class_name => "UR::Entity",
-            #        inheritance_priority => 0,
-            #    );
-            #    Carp::confess("Failed to generate inheritance link!?") unless $inheritance;
-            #}
         }
     } # next table
 
@@ -921,30 +857,18 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     $self->status_message($_) foreach @saved_removed_column_messages;
 
     # COLUMN
-    my @column_property_translations = (
-        ['data_type'    => 'data_type'],
-        ['data_length'  => 'data_length'],
-        ['nullable'     => 'is_optional', sub { (defined($_[0]) and ($_[0] eq "Y")) ? 1 : 0 } ],
-        ['remarks'      => 'doc'],
-    );
     
     for my $column (sort $sorter @{ $dd_changes_by_class{'UR::DataSource::RDBMS::TableColumn'} }) {
         my $table = $column->get_table;
         my $column_name = $column->column_name;
         my $data_source = $table->data_source;
-        my($ur_data_type,$default_length) = @{ $data_source->ur_data_type_for_data_source_data_type($column->data_type) };
+        my($ur_data_type, $default_length) = @{ $data_source->ur_data_type_for_data_source_data_type($column->data_type) };
         my $ur_data_length = defined($column->data_length) ? $column->data_length : $default_length;
 
-        #my $class = UR::Object::Type->get(
-        #    data_source => $table->data_source,
-        #    table_name => $table->table_name,
-        #);
-        #my $class = $data_source->get_class_meta_for_table($table);
         my $class = $self->_get_class_meta_for_table_name(data_source => $data_source,
                                                           table_name => $table->table_name);
 
         unless ($class) {
-            ##$DB::single = 1;
             $class = $self->_get_class_meta_for_table_name(data_source => $data_source,
                                                           table_name => $table->table_name);
             Carp::confess("Class object missing for table " . $table->table_name) unless $class;
@@ -952,7 +876,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class_name = $class->class_name;
         my $property;
         foreach my $prop_object ( $class->direct_property_metas ) {
-            if (defined $prop_object->column_name and $prop_object->column_name eq $column_name) {
+            if (defined $prop_object->column_name and lc($prop_object->column_name) eq lc($column_name)) {
                 $property = $prop_object;
                 last;
             }
@@ -960,6 +884,21 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
         # We care less whether the column is new/updated, than whether there is property metadata for it.
         if ($property) {
+            my @column_property_translations = (
+                # [ column_name, property_name, conversion_sub(column_obj, value) ]
+                ['data_length'  => 'data_length',
+                    # lengths for these data types are based on the number of bytes used internally in the
+                    # database.  The UR-based objects will store the text version, which will always be longer,
+                    # making $obj->__errors__() complain about the length being out of bounds
+                    sub { my ($c, $av) = @_;  defined($av) ? $av : ($c->is_time_data ? undef : $ur_data_length) } ],
+                ['data_type'    => 'data_type',
+                    sub { my ($c, $av) = @_;  defined($ur_data_type) ? $ur_data_type : $av } ],
+                ['nullable'     => 'is_optional',
+                    sub { my ($c, $av) = @_; (defined($av) and ($av eq "Y")) ? 1 : 0 } ],
+                ['remarks'      => 'doc',
+                    # Ideally this would only use DB value ($av) if the last_ddl_time was newer.
+                    sub { my ($c, $av) = @_; defined($av) ? $av : $property->doc } ],
+            );
             # update
             for my $translation (@column_property_translations) {
                 my ($column_attr, $property_attr, $conversion_sub) = @$translation;
@@ -968,22 +907,14 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 no warnings;
                 if (UR::Context->_get_committed_property_value($column,$column_attr) ne $column->$column_attr) {
                     if ($conversion_sub) {
-                        $property->$property_attr($conversion_sub->($column->$column_attr));
+                        $property->$property_attr($conversion_sub->($column, $column->$column_attr));
                     }
                     else {
                         $property->$property_attr($column->$column_attr);
                     }
                 }
             }
-            $property->data_type($ur_data_type) if (! defined $property->data_type);
-            # lengths for these data types are based on the number of bytes used internally in the
-            # database.  The UR-based objects will store the text version, which will always be longer,
-            # making $obj->__errors__() complain about the length being out of bounds
-            $property->data_length($column->is_time_data ? undef : $ur_data_length) if (! defined $property->data_length);
 
-            $property->is_optional($column->nullable eq "Y" ? 1 : 0);
-            $property->doc($column->remarks);
-            
             if ($property->__changes__) {
                 no warnings;
                 $self->status_message(
@@ -1007,23 +938,35 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 );
             }
 
-            $property = UR::Object::Property->create(
-                class_name     => $class_name,
-                property_name  => $property_name,
-                column_name    => $column_name,
-                data_type      => $ur_data_type,
-                data_length    => $ur_data_length,
-                is_optional    => $column->nullable eq "Y" ? 1 : 0,
-                is_volatile    => 0,
-                doc            => $column->remarks,
-                is_specified_in_module_header => 1, 
-            );
+            my $create_exception;
+            for (my $attempt = 0; $attempt < 3; $attempt++) {
+                $property_name = '_' . $property_name if $attempt;
 
-            no warnings;
+                $create_exception = do {
+                    local $@;
+                    eval {
+                        $property = UR::Object::Property->create(
+                            class_name     => $class_name,
+                            property_name  => $property_name,
+                            column_name    => $column_name,
+                            data_type      => $ur_data_type,
+                            data_length    => $ur_data_length,
+                            is_optional    => $column->nullable eq "Y" ? 1 : 0,
+                            is_volatile    => 0,
+                            doc            => $column->remarks,
+                            is_specified_in_module_header => 1,
+                        );
+                    };
+                    $@;
+                };
+                last if $property;
+            }
+
+            no warnings 'uninitialized';
             $self->status_message(
                 sprintf("A %-40s property %-16s for column %s.%s (%s %s)\n",
                                                         $class_name,
-                                                        $property_name,
+                                                        $property->property_name,
                                                         $table->table_name, 
                                                         $column_name,
                                                         $column->data_type,
@@ -1031,10 +974,14 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             );
             
             unless ($property) {
-                Carp::confess(
-                        "Failed to create property $property_name on class $class_name. "
-                        . UR::Object::Property->error_message
-                );
+                if ($create_exception =~ m/An object of class UR::Object::Property already exists/) {
+                    $self->warning_message("Conflicting property names already exist in class $class_name for column $column_name in table ".$table->table_name);
+                } else {
+                    Carp::confess(
+                            "Failed to create property $property_name on class $class_name. "
+                            . UR::Object::Property->error_message
+                    );
+                }
             }
         }
     } # next column
@@ -1122,18 +1069,11 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
     $self->status_message("Updating class unique constraints...\n");
 
-    ##$DB::single = 1;
-
     # UNIQUE CONSTRAINT / UNIQUE INDEX -> UNIQUE GROUP (loop table objecs since we have no PK DD objects)
     for my $table (sort $sorter @{ $dd_changes_by_class{'UR::DataSource::RDBMS::Table'} }) {
         # created/updated/unchanged
         # delete and re-create
 
-        #my $class = UR::Object::Type->get(
-        #    data_source => $table->data_source,
-        #    table_name => $table->table_name,
-        #);
-        #my $class = $table->__meta__();
         my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
                                                           table_name => $table->table_name);
         my $class_name = $class->class_name;
@@ -1146,7 +1086,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             eval { $class->remove_unique_constraint($uc_name) };
             if ($@ =~ m/There is no constraint named/) {
                 next;  # it's OK if there's no UR metadata for this constraint yet
-            } else {
+            } elsif ($@) {
                 die $@;
             }
 
