@@ -8,7 +8,7 @@ require UR;
 use Scalar::Util qw(looks_like_number);
 
 our @ISA = ('UR::ModuleBase');
-our $VERSION = "0.43"; # UR $VERSION;;
+our $VERSION = "0.44"; # UR $VERSION;;
 
 # Base object API
 
@@ -28,6 +28,33 @@ sub delete {
     $UR::Context::current->delete_entity(@_);
 }
 
+sub copy {
+    my $self = shift;
+    my %override = @_;
+
+    my $meta = $self->__meta__;
+    my @copyable_properties =
+        grep { !$_->is_delegated && !$_->is_id }
+        $meta->properties;
+
+    my %params;
+    for my $p (@copyable_properties) {
+        my $name = $p->property_name;
+        if ($p->is_many) {
+            if (my @value = $self->$name) {
+                $params{$name} = \@value;
+            }
+        }
+        else {
+            if (defined(my $value = $self->$name)) {
+                $params{$name} = $value;
+            }
+        }
+    }
+
+    return $self->class->create(%params, %override);
+}
+
 
 # Meta API
 
@@ -301,18 +328,21 @@ sub define_boolexpr {
 sub define_set {
     my $class = shift;
     $class = ref($class) || $class;
-    my $rule = UR::BoolExpr->resolve($class,@_);
+    my $rule = UR::BoolExpr->resolve_normalized($class,@_);
+    my $flattened_rule = $rule->flatten_hard_refs();
     my $set_class = $class . "::Set";
-    return $set_class->get($rule->id);
+    return $set_class->get($flattened_rule->id);
 }
 
 sub add_observer {
     my $self = shift;
     my %params = @_;
 
+    if (ref($self)) {
+        $params{subject_id} = $self->id;
+    }
     my $observer = UR::Observer->create(
         subject_class_name => $self->class,
-        subject_id => (ref($self) ? $self->id : undef),
         %params,
     );
     unless ($observer) {
@@ -615,6 +645,11 @@ sub __extend_namespace__ {
 
 # Handling of references within the current process
 
+sub is_weakened {
+    my $self = shift;
+    return (exists $self->{__weakened} && $self->{__weakened});
+}
+
 sub __weaken__ {
     # Mark this object as unloadable by the object cache pruner.
     # If the class has a data source, then a weakened object is dropped
@@ -627,13 +662,80 @@ sub __weaken__ {
     $self->{'__weakened'} = 1;
 }
 
+sub is_strengthened {
+    my $self = shift;
+    return (exists $self->{__strengthened} && $self->{__strengthened});
+}
+
 sub __strengthen__ {
     # Indicate this object should never be unloaded by the object cache pruner
+    # or AutoUnloadPool
     my $self = $_[0];
     delete $self->{'__weakened'};
     $self->{'__strengthened'} = 1;
 }
 
+sub is_prunable {
+    my $self = shift;
+    return 0 if $self->is_strengthened;
+    return 1 if $self->is_weakened;
+    return 0 if $self->__meta__->is_meta;
+    return 0 if $self->{__get_serial} && $self->__changes__ && @{[$self->__changes__]};
+    return 1;
+}
+
+
+sub __rollback__ {
+    my $self = shift;
+
+    my $saved = $self->{db_saved_uncommitted} || $self->{db_committed};
+    unless ($saved) {
+        return UR::Object::delete($self);
+    }
+
+    my $meta = $self->__meta__;
+
+    my $should_rollback = sub {
+        my $property_meta = shift;
+        return ! (
+            defined $property_meta->is_id
+            || ! defined $property_meta->column_name
+            || $property_meta->is_delegated
+            || $property_meta->is_legacy_eav
+            || ! $property_meta->is_mutable
+            || $property_meta->is_transient
+            || $property_meta->is_constant
+        );
+    };
+    my @rollback_property_names =
+        map { $_->property_name }
+        grep { $should_rollback->($_) }
+        map { $meta->property_meta_for_name($_) }
+        $meta->all_property_names;
+
+    # Existing object.  Undo all changes since last sync, or since load
+    # occurred when there have been no syncs.
+    foreach my $property_name ( @rollback_property_names ) {
+        $self->__rollback_property__($property_name);
+    }
+
+    delete $self->{'_change_count'};
+
+    return $self;
+}
+
+
+sub __rollback_property__ {
+    my ($self, $property_name) = @_;
+    my $saved = $self->{db_saved_uncommitted} || $self->{db_committed};
+    unless ($saved) {
+        Carp::croak(qq(Cannot rollback property '$property_name' because it has no saved state));
+    }
+    my $saved_value = UR::Context->current->value_for_object_property_in_underlying_context($self, $property_name);
+    return $self->$property_name($saved_value);
+}
+
+
 sub DESTROY {
     # Handle weak references in the object cache.
     my $obj = shift;
@@ -898,6 +1000,15 @@ and a fresh reference will later be returnable by get().  See the documentation
 for details on how deleted objects are rememberd and removed later from the database, and how
 deleted objects are re-constructed on STM rollback.
 
+=item copy
+
+  $obj->copy(%overrides)
+
+Copies the existing C<$obj> by copying the values of all direct properties,
+except for ID properties, to a newly created object of the same type.  A list
+of params and values may be provided as overrides to the existing values or to
+specify an ID.
+
 =item class
 
  $class_name = $obj->class;
@@ -1155,6 +1266,8 @@ Adds an observer to an object, monitoring one or more of its properties for chan
 
 The specified callback is fired upon property changes which match the observation request.
 
+See L<UR::Observer> for details.
+
 =item create_mock
 
  $mock = SomeClass->create_mock(
@@ -1316,6 +1429,10 @@ the cache, and will not be considered for pruning.
 
 See L<UR::Context> for more information about the pruning mechanism.
 
+=item is_strengthened
+
+Check if an object has been stengthened, C<__stengthen__>.
+
 =item __weaken__
 
   $obj->__weaken__();
@@ -1324,6 +1441,10 @@ Give a hint to the object cache pruner that this instance is not going to be use
 in the application in the future, and should be removed with preference when
 pruning the cache.
 
+=item is_weakened
+
+Check if an object has been weakened, C<__weaken__>.
+
 =item DESTROY
 
 Perl calls this method on any object before garbage collecting it.  It
