@@ -9,8 +9,9 @@ use UR;
 use UR::Object::Command::List::Style;
 use List::Util qw(reduce);
 use Command::V2;
+use Carp qw();
 
-our $VERSION = "0.43"; # UR $VERSION;
+our $VERSION = "0.44"; # UR $VERSION;
 
 class UR::Object::Command::List {
     is => 'Command::V2',
@@ -79,7 +80,6 @@ sub sub_command_sort_position { .2 };
 sub create {
     my $class = shift;
     my $self = $class->SUPER::create(@_);
-	#$DB::single = 1;
 
     if (defined($self->csv_delimiter)
         and ($self->csv_delimiter ne $self->__meta__->property_meta_for_name('csv_delimiter')->default_value)
@@ -101,42 +101,22 @@ sub create {
 sub _resolve_boolexpr {
     my $self = shift;
 
-    my ($bool_expr,%extra);
-    eval {
-        ($bool_expr, %extra) = UR::BoolExpr->resolve_for_string(
-                                   $self->subject_class_name,
-                                   $self->_complete_filter,
-                                   $self->_hint_string,
-                                   $self->order_by,
-                               );
-    };
-    my $error = $@;
-
-    unless ($bool_expr) {
-        eval {
-            ($bool_expr, %extra) = UR::BoolExpr->_old_resolve_for_string(
-                                   $self->subject_class_name,
-                                   $self->_complete_filter,
-                                   $self->_hint_string,
-                                   $self->order_by,
-                           )
-        };
-        if ($bool_expr) {
-            $self->warning_message("Failed to parse your query, but it was recognized by the deprecated filter parser.\n  Try putting quotes around the entire filter expression.\n  Use double quotes if your filter already includes single quotes, and vice-versa.\n  Values containing spaces need quotes around them as well\n  The error from the parser was:\n    $error");
-        } else {
-            die $error if $error;
-        }
-    }
+    my ($bool_expr, %extra) = UR::BoolExpr->resolve_for_string(
+        $self->subject_class_name,
+        $self->_complete_filter,
+        $self->_hint_string,
+        $self->order_by,
+    );
 
     if (%extra) {
-        $self->error_message( 
+        Carp::croak(
             sprintf(
                 'Cannot list for class %s because some items in the filter or show were not properties of that class: %s',
-                $self->subject_class_name, 
+                $self->subject_class_name,
                 join(', ', keys %extra)
             )
-        )
-    } 
+        );
+    }
 
     return $bool_expr;
 }
@@ -161,7 +141,7 @@ sub execute {
     my ($ns) = ($subject_class_name =~ /^(.*?)::/);
     eval "use $ns";
     my $subject_class = UR::Object::Type->get($subject_class_name);
-    
+
     # Determine things to show
     my @fields = $self->_resolve_field_list;
 
