@@ -7,7 +7,7 @@ require UR;
 use Lingua::EN::Inflect;
 use Class::AutoloadCAN;
 
-our $VERSION = "0.43"; # UR $VERSION;;
+our $VERSION = "0.44"; # UR $VERSION;;
 our @CARP_NOT = qw( UR::DataSource::RDBMS UR::Object::Type );
 
 # class_meta and r_class_meta duplicate the functionality if two properties of the same name,
@@ -226,7 +226,7 @@ sub _involves_id_property {
 
     if ($self->via) {
         my $via_meta = $self->via_property_meta;
-        return 1 if ($via_meta and $via_meta->_involves_id_property);
+        return 1 if ($via_meta and $via_meta ne $self and $via_meta->_involves_id_property);
 
         if ($self->to) {
             my $to_meta = $self->to_property_meta;
@@ -341,9 +341,13 @@ sub _get_direct_join_linkage {
 
         my @my_id_by = @{ $self->id_by };
         my @their_id_by = @{ $r_class_meta->{'id_by'} };
-        unless (@their_id_by) {
+        if (! @their_id_by
+            or
+            (@my_id_by == 1 and @their_id_by > 1)
+        ) {
             @their_id_by = ( 'id' );
         }
+
         unless (@my_id_by == @their_id_by) {
             Carp::croak("Property '" . $self->property_name . "' of class '" . $self->class_name . "' "
                         . "has " . scalar(@my_id_by) . " id_by elements, while its data_type ("
@@ -365,14 +369,9 @@ sub _get_direct_join_linkage {
 
 sub _resolve_join_chain {
     my $self = shift;
-    my $join_label = shift;
-
-    $join_label ||= $self->property_name;
-
     return UR::Object::Join->resolve_chain(
         $self->class_name,
         $self->property_name,
-        $join_label,
     );
 }
 
