@@ -3,7 +3,7 @@ use warnings;
 use strict;
 
 require UR;
-our $VERSION = "0.43"; # UR $VERSION;
+our $VERSION = "0.44"; # UR $VERSION;
 
 use Sys::Hostname;
 use Cwd;
@@ -994,19 +994,23 @@ sub _load {
     }
 
     # Check the filesystem.  The file may create its metadata object.
-    eval "use $class_name";
-    unless ($@) {
+    my $exception = do {
+        local $@;
+        eval "use $class_name";
+        $@;
+    };
+    unless ($exception) {
         # If the above module was loaded, and is an UR::Object,
         # this will find the object.  If not, it will return nothing.
         $class_obj = $UR::Context::current->get_objects_for_class_and_rule($class,$rule,0);
         return $class_obj if $class_obj;
     }
-    if ($@) {
+    if ($exception) {
         # We need to handle $@ here otherwise we'll see
         # "Can't locate UR/Object/Type/Ghost.pm in @INC" error.
         # We want to fall through "in the right circumstances".
         (my $module_path = $class_name . '.pm') =~ s/::/\//g;
-        Carp::croak("Error while autoloading with 'use $class_name': $@") unless ($@ =~ /Can't locate $module_path in \@INC/);
+        Carp::croak("Error while autoloading with 'use $class_name': $exception") unless ($exception =~ /Can't locate $module_path in \@INC/);
         # FIXME: I think other conditions here will result in silent errors.
     }
 
@@ -1438,6 +1442,7 @@ sub _property_change_callback {
     my($property_obj,$method, $old_val, $new_val) = @_;
 
     return if ($method eq 'load' || $method eq 'unload');
+    return unless ref($property_obj);  # happens when, say, error_message is called on the UR::Object::Property class
 
     my $class_obj = UR::Object::Type->get(class_name => $property_obj->class_name);
     my $property_name = $property_obj->property_name;
@@ -1470,7 +1475,8 @@ sub _property_change_callback {
         &_id_property_change_callback($property_obj, $change);
     }
 
-    if (exists $class_obj->{'has'}->{$property_name}->{$method}) {
+    if (exists $class_obj->{'has'}->{$property_name}
+        && exists $class_obj->{'has'}->{$property_name}->{$method}) {
         $class_obj->{'has'}->{$property_name}->{$method} = $new_val;
 
     } 
@@ -1575,19 +1581,15 @@ sub __signal_change__ {
     return @rv;
 }
 
-our %STANDARD_VALID_SIGNALS = ( create        => 1,
-                                'delete'      => 1,
-                                commit        => 1,
-                                rollback      => 1,
-                                load          => 1,
-                                unload        => 1,
-                                load_external => 1 );
+my @default_valid_signals = qw(create delete commit rollback load unload load_external subclass_loaded);
+our %STANDARD_VALID_SIGNALS;
+@STANDARD_VALID_SIGNALS{@default_valid_signals} = (1) x @default_valid_signals;
 sub _is_valid_signal {
     my $self = shift;
     my $aspect = shift;
 
-    # Undefined attributes indicate that the subscriber wants any changes at all to generate a callback.
-    return 1 if (! defined $aspect);
+    # An aspect of empty string (or undef) means all aspects are being observed.
+    return 1 unless (defined($aspect) and length($aspect));
 
     # All standard creation and destruction methods emit a signal.
     return 1 if ($STANDARD_VALID_SIGNALS{$aspect});
@@ -1642,6 +1644,67 @@ sub ungenerate {
     delete $INC{$module_name};    
     $self->{'generated'} = 0;
 }
-    
+
+sub singular_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+    unless (exists $self->{_accessor_singular_names}->{$property_name}) {
+        my $property_meta = $self->property_meta_for_name($property_name) if ($self->generated);
+        if ($bootstrapping  # trust the caller when bootstrapping
+            or
+            ! $self->generated # when called from UR::Object::Type::AccessorWriter and the property isn't created yet
+            or
+            ($property_meta && $property_meta->is_many)
+        ) {
+            require Lingua::EN::Inflect;
+            $self->{_accessor_singular_names}->{$property_name} = Lingua::EN::Inflect::PL_V($property_name);
+        } else {
+            $self->{_accessor_singular_names}->{$property_name} = undef;
+        }
+    }
+    return $self->{_accessor_singular_names}->{$property_name};
+}
+
+sub iterator_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_iterator";
+}
+
+sub set_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_set";
+}
+
+sub rule_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "__${singular}_rule";
+}
+
+sub arrayref_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_arrayref";
+}
+
+sub adder_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "add_${singular}";
+}
+
+sub remover_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "remove_${singular}";
+}
+
 1;
 
